@@ -6,11 +6,21 @@ from pydantic_core import CoreSchema, core_schema
 
 from ..errors import PydanticUserError
 from . import _core_utils
-from ._core_utils import CoreSchemaField, collect_definitions
+from ._core_utils import NEEDS_APPLY_DISCRIMINATED_UNION_METADATA_KEY, CoreSchemaField, collect_definitions
 
 CORE_SCHEMA_METADATA_DISCRIMINATOR_PLACEHOLDER_KEY = 'pydantic.internal.union_discriminator'
 
 
+class MissingDefinitionForUnionRef(Exception):
+    """Raised when applying a discriminated union discriminator to a schema
+    requires a definition that is not yet defined
+    """
+
+    def __init__(self, ref: str) -> None:
+        self.ref = ref
+        super().__init__(f'Missing definition for ref {self.ref!r}')
+
+
 def set_discriminator(schema: CoreSchema, discriminator: Any) -> None:
     schema.setdefault('metadata', {})
     metadata = schema.get('metadata')
@@ -19,9 +29,14 @@ def set_discriminator(schema: CoreSchema, discriminator: Any) -> None:
 
 
 def apply_discriminators(schema: core_schema.CoreSchema) -> core_schema.CoreSchema:
-    definitions = collect_definitions(schema)
+    definitions: dict[str, CoreSchema] | None = None
 
     def inner(s: core_schema.CoreSchema, recurse: _core_utils.Recurse) -> core_schema.CoreSchema:
+        nonlocal definitions
+        if 'metadata' in s:
+            if s['metadata'].get(NEEDS_APPLY_DISCRIMINATED_UNION_METADATA_KEY, True) is False:
+                return s
+
         s = recurse(s, inner)
         if s['type'] == 'tagged-union':
             return s
@@ -29,6 +44,8 @@ def apply_discriminators(schema: core_schema.CoreSchema) -> core_schema.CoreSche
         metadata = s.get('metadata', {})
         discriminator = metadata.get(CORE_SCHEMA_METADATA_DISCRIMINATOR_PLACEHOLDER_KEY, None)
         if discriminator is not None:
+            if definitions is None:
+                definitions = collect_definitions(schema)
             s = apply_discriminator(s, discriminator, definitions)
         return s
 
@@ -53,7 +70,7 @@ def apply_discriminator(
             - If `discriminator` is used with invalid union variant.
             - If `discriminator` is used with `Union` type with one variant.
             - If `discriminator` value mapped to multiple choices.
-        ValueError:
+        MissingDefinitionForUnionRef:
             If the definition for ref is missing.
         PydanticUserError:
             - If a model in union doesn't have a discriminator field.
@@ -188,7 +205,8 @@ class _ApplyInferredDiscriminator:
             schema = core_schema.union_schema([schema])
 
         # Reverse the choices list before extending the stack so that they get handled in the order they occur
-        self._choices_to_handle.extend(schema['choices'][::-1])
+        choices_schemas = [v[0] if isinstance(v, tuple) else v for v in schema['choices'][::-1]]
+        self._choices_to_handle.extend(choices_schemas)
         while self._choices_to_handle:
             choice = self._choices_to_handle.pop()
             self._handle_choice(choice)
@@ -237,10 +255,11 @@ class _ApplyInferredDiscriminator:
             self._handle_choice(choice['schema'])  # unwrap the nullable schema
         elif choice['type'] == 'union':
             # Reverse the choices list before extending the stack so that they get handled in the order they occur
-            self._choices_to_handle.extend(choice['choices'][::-1])
+            choices_schemas = [v[0] if isinstance(v, tuple) else v for v in choice['choices'][::-1]]
+            self._choices_to_handle.extend(choices_schemas)
         elif choice['type'] == 'definition-ref':
             if choice['schema_ref'] not in self.definitions:
-                raise ValueError(f"Missing definition for ref {choice['schema_ref']!r}")
+                raise MissingDefinitionForUnionRef(choice['schema_ref'])
             self._handle_choice(self.definitions[choice['schema_ref']])
         elif choice['type'] not in {
             'model',
@@ -315,7 +334,8 @@ class _ApplyInferredDiscriminator:
         elif choice['type'] == 'union':
             values = []
             for subchoice in choice['choices']:
-                subchoice_values = self._infer_discriminator_values_for_choice(subchoice, source_name=None)
+                subchoice_schema = subchoice[0] if isinstance(subchoice, tuple) else subchoice
+                subchoice_values = self._infer_discriminator_values_for_choice(subchoice_schema, source_name=None)
                 values.extend(subchoice_values)
             return values
 
@@ -341,9 +361,8 @@ class _ApplyInferredDiscriminator:
         elif choice['type'] == 'definition-ref':
             schema_ref = choice['schema_ref']
             if schema_ref not in self.definitions:
-                raise ValueError(f'Missing definition for inner ref {schema_ref!r}')
+                raise MissingDefinitionForUnionRef(schema_ref)
             return self._infer_discriminator_values_for_choice(self.definitions[schema_ref], source_name=source_name)
-
         else:
             raise TypeError(
                 f'{choice["type"]!r} is not a valid discriminated union variant;'
@@ -422,7 +441,8 @@ class _ApplyInferredDiscriminator:
             # For example, this lets us handle `Union[Literal['key'], Union[Literal['Key'], Literal['KEY']]]`
             values: list[Any] = []
             for choice in schema['choices']:
-                choice_values = self._infer_discriminator_values_for_inner_schema(choice, source)
+                choice_schema = choice[0] if isinstance(choice, tuple) else choice
+                choice_values = self._infer_discriminator_values_for_inner_schema(choice_schema, source)
                 values.extend(choice_values)
             return values
 
