@@ -877,7 +877,7 @@ def test_missing_discriminator_field() -> None:
 def test_wrap_function_schema() -> None:
     cat_fields = {'kind': core_schema.typed_dict_field(core_schema.literal_schema(['cat']))}
     dog_fields = {'kind': core_schema.typed_dict_field(core_schema.literal_schema(['dog']))}
-    cat = core_schema.general_wrap_validator_function(lambda x, y, z: None, core_schema.typed_dict_schema(cat_fields))
+    cat = core_schema.with_info_wrap_validator_function(lambda x, y, z: None, core_schema.typed_dict_schema(cat_fields))
     dog = core_schema.typed_dict_schema(dog_fields)
     schema = core_schema.union_schema([cat, dog])
 
@@ -885,7 +885,7 @@ def test_wrap_function_schema() -> None:
         'choices': {
             'cat': {
                 'function': {
-                    'type': 'general',
+                    'type': 'with-info',
                     'function': HasRepr(IsStr(regex=r'<function [a-z_]*\.<locals>\.<lambda> at 0x[0-9a-fA-F]+>')),
                 },
                 'schema': {
@@ -915,7 +915,7 @@ def test_plain_function_schema_is_invalid() -> None:
     ):
         apply_discriminator(
             core_schema.union_schema(
-                [core_schema.general_plain_validator_function(lambda x, y: None), core_schema.int_schema()]
+                [core_schema.with_info_plain_validator_function(lambda x, y: None), core_schema.int_schema()]
             ),
             'kind',
         )
@@ -931,7 +931,7 @@ def test_invalid_str_choice_discriminator_values() -> None:
             # NOTE: Wrapping the union with a validator results in failure to more thoroughly decompose the tagged
             # union. I think this would be difficult to avoid in the general case, and I would suggest that we not
             # attempt to do more than this until presented with scenarios where it is helpful/necessary.
-            core_schema.general_wrap_validator_function(lambda x, y, z: x, dog),
+            core_schema.with_info_wrap_validator_function(lambda x, y, z: x, dog),
         ]
     )
 
@@ -1017,7 +1017,7 @@ def test_wrapped_nullable_union() -> None:
             # NOTE: Wrapping the union with a validator results in failure to more thoroughly decompose the tagged
             # union. I think this would be difficult to avoid in the general case, and I would suggest that we not
             # attempt to do more than this until presented with scenarios where it is helpful/necessary.
-            core_schema.general_wrap_validator_function(
+            core_schema.with_info_wrap_validator_function(
                 lambda x, y, z: x, core_schema.nullable_schema(core_schema.union_schema([cat, dog]))
             ),
         ]
@@ -1055,7 +1055,7 @@ def test_wrapped_nullable_union() -> None:
                 'cat': {
                     'type': 'function-wrap',
                     'function': {
-                        'type': 'general',
+                        'type': 'with-info',
                         'function': HasRepr(IsStr(regex=r'<function [a-z_]*\.<locals>\.<lambda> at 0x[0-9a-fA-F]+>')),
                     },
                     'schema': {
@@ -1088,7 +1088,7 @@ def test_wrapped_nullable_union() -> None:
                 'dog': {
                     'type': 'function-wrap',
                     'function': {
-                        'type': 'general',
+                        'type': 'with-info',
                         'function': HasRepr(IsStr(regex=r'<function [a-z_]*\.<locals>\.<lambda> at 0x[0-9a-fA-F]+>')),
                     },
                     'schema': {
