@@ -406,6 +406,7 @@ def test_caches_get_cleaned_up_with_aliased_parametrized_bases(clean_cache):
 
 
 @pytest.mark.skipif(platform.python_implementation() == 'PyPy', reason='PyPy does not play nice with PyO3 gc')
+@pytest.mark.skipif(sys.version_info[:2] == (3, 9), reason='The test randomly fails on Python 3.9')
 def test_circular_generic_refs_get_cleaned_up():
     initial_cache_size = len(_GENERIC_TYPES_CACHE)
 
@@ -763,6 +764,7 @@ def test_partial_specification_with_inner_typevar():
     assert nested_resolved.b == [456]
 
 
+@pytest.mark.skipif(sys.version_info < (3, 12), reason='repr different on older versions')
 def test_partial_specification_name():
     AT = TypeVar('AT')
     BT = TypeVar('BT')
@@ -772,7 +774,7 @@ def test_partial_specification_name():
         b: BT
 
     partial_model = Model[int, BT]
-    assert partial_model.__name__ == 'Model[int, ~BT]'
+    assert partial_model.__name__ == 'Model[int, TypeVar]'
     concrete_model = partial_model[str]
     assert concrete_model.__name__ == 'Model[int, str]'
 
@@ -1782,7 +1784,7 @@ def test_generic_recursive_models_with_a_concrete_parameter(create_module):
     M1 = module.M1
 
     # assert M1.__pydantic_core_schema__ == {}
-    assert collect_invalid_schemas(M1.__pydantic_core_schema__) == []
+    assert collect_invalid_schemas(M1.__pydantic_core_schema__) is False
 
 
 def test_generic_recursive_models_complicated(create_module):
@@ -1842,7 +1844,7 @@ def test_generic_recursive_models_complicated(create_module):
 
     M1 = module.M1
 
-    assert collect_invalid_schemas(M1.__pydantic_core_schema__) == []
+    assert collect_invalid_schemas(M1.__pydantic_core_schema__) is False
 
 
 def test_generic_recursive_models_in_container(create_module):
@@ -1863,11 +1865,6 @@ def test_generic_recursive_models_in_container(create_module):
     assert type(instance.foobar[0]) == MyGenericModel[int]
 
 
-def test_schema_is_valid():
-    assert not collect_invalid_schemas(core_schema.none_schema())
-    assert collect_invalid_schemas(core_schema.nullable_schema(core_schema.int_schema(metadata={'invalid': True})))
-
-
 def test_generic_enum():
     T = TypeVar('T')
 
@@ -2613,3 +2610,15 @@ def test_no_generic_base():
             'type': 'int_parsing',
         },
     ]
+
+
+def test_reverse_order_generic_hashability():
+    T = TypeVar('T')
+
+    class Model(Generic[T], BaseModel):
+        x: T
+        model_config = dict(frozen=True)
+
+    m1 = Model[int](x=1)
+    m2 = Model[int](x=1)
+    assert len({m1, m2}) == 1
